/*
 * Xen domain builder -- ia64 bits.
 *
 * Most architecture-specific code for ia64 goes here.
 *   - fill architecture-specific structs.
 *
 * This code is licenced under the GPL.
 * written 2006 by Gerd Hoffmann <kraxel@suse.de>.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <inttypes.h>
#include <assert.h>
/* #include <asm/kregs.h> */

#include <xen/xen.h>
#include <xen/foreign/ia64.h>
#include <xen/io/protocols.h>

#include "xg_private.h"
#include "xc_dom.h"
#include "xenctrl.h"

extern int start_info_mips(struct xc_dom_image *dom);
extern int shared_info_mips(struct xc_dom_image *dom, void *ptr);

/* ------------------------------------------------------------------------ */

static int alloc_magic_pages(struct xc_dom_image *dom)
{
    xc_dom_printf("%s\n", __FUNCTION__);

    return 0;
}

int start_info_mips(struct xc_dom_image *dom)
{
    xc_dom_printf("%s\n", __FUNCTION__);

    return 0;
}

int shared_info_mips(struct xc_dom_image *dom, void *ptr)
{
    xc_dom_printf("%s\n", __FUNCTION__);

    return 0;
}

static int vcpu_mips(struct xc_dom_image *dom, void *ptr)
{
    xc_dom_printf("%s: called\n", __FUNCTION__);

    return 0;
}

/* ------------------------------------------------------------------------ */

static struct xc_dom_arch xc_dom_arch = {
    .guest_type = "xen-3.0-mips",
    .native_protocol = XEN_IO_PROTO_ABI_MIPS,
    .page_shift = PAGE_SHIFT_MIPS,
    .alloc_magic_pages = alloc_magic_pages,
    .start_info = start_info_mips,
    .shared_info = shared_info_mips,
    .vcpu = vcpu_mips,
};

static void __init register_arch_hooks(void)
{
    xc_dom_register_arch_hooks(&xc_dom_arch);
}

#include "xc_efi.h"

int arch_setup_meminit(struct xc_dom_image *dom)
{
    xen_pfn_t pfn;
    int rc;
    unsigned long start;
    unsigned long nbr;

    /* setup initial p2m */
    start = 0;
    nbr = dom->total_pages;

    /* setup initial p2m */
    dom->p2m_host = xc_dom_malloc(dom, sizeof(xen_pfn_t) * nbr);
    for ( pfn = 0; pfn < nbr; pfn++ )
        dom->p2m_host[pfn] = start + pfn;

    /* allocate guest memory */
    rc = xc_domain_memory_populate_physmap(dom->guest_xc, dom->guest_domid,
                                           nbr, 0, 0,
                                           dom->p2m_host);
    return rc;
}

int arch_setup_bootearly(struct xc_dom_image *dom)
{
    DECLARE_DOMCTL;
    int rc;

    xc_dom_printf("%s: setup firmware for %s\n", __FUNCTION__, dom->guest_type);

    memset(&domctl, 0, sizeof(domctl));
    domctl.cmd = XEN_DOMCTL_arch_setup;
    domctl.domain = dom->guest_domid;
    domctl.u.arch_setup.flags = XEN_DOMAINSETUP_query;
    rc = do_domctl(dom->guest_xc, &domctl);
    if (rc)
        return rc;

    memset(&domctl, 0, sizeof(domctl));
    domctl.cmd = XEN_DOMCTL_arch_setup;
    domctl.domain = dom->guest_domid;
    domctl.u.arch_setup.flags = 0;

    rc = do_domctl(dom->guest_xc, &domctl);
    return rc;
}

int arch_setup_bootlate(struct xc_dom_image *dom)
{
    unsigned int page_size = XC_DOM_PAGE_SIZE(dom);
    shared_info_t *shared_info;

    /* setup shared_info page */
    xc_dom_printf("%s: shared_info: mfn 0x%" PRIpfn "\n",
                  __FUNCTION__, dom->shared_info_mfn);
    shared_info = xc_map_foreign_range(dom->guest_xc, dom->guest_domid,
                                       page_size,
                                       PROT_READ | PROT_WRITE,
                                       dom->shared_info_mfn);
    if ( shared_info == NULL )
        return -1;
    dom->arch_hooks->shared_info(dom, shared_info);
    munmap(shared_info, page_size);
    return 0;
}

/*
 * Local variables:
 * mode: C
 * c-set-style: "BSD"
 * c-basic-offset: 4
 * tab-width: 4
 * indent-tabs-mode: nil
 * End:
 */
