#include <xen/config.h>
#include <asm/asm.h>
#include <asm/asmmacro.h>
#include <asm/irqflags.h>
#include <asm/regdef.h>
#include <asm/page.h>
#include <asm/mipsregs.h>
#include <asm/stackframe.h>
#include <asm/isadep.h>

#if (_MIPS_SIM == MIPS_SIM_ABI64)
#define LW ld
#else
#define LW lw
#endif

#ifdef XLP_SIM
#define la dla
#undef LA
#define LA dla
#endif

NESTED(except_vec_tlbrefill, 0, sp)
	wait
1:	b	1b
	nop
    nop
END(except_vec_tlbrefill)

NESTED(except_vec_xtlbrefill, 0, sp)
	wait
1:	b	1b
	nop
    nop
END(except_vec_xtlbrefill)

NESTED(except_vec_cacheerror, 0, sp)
	wait
1:	b	1b
	nop
    nop
END(except_vec_cacheerror)

NESTED(except_vec_vecint, 0, sp)
	wait
1:	b	1b
	nop
    nop
END(except_vec_vecint)


#define resume_xen restore_all

	.text
	.align 5
FEXPORT(ret_from_exception)
	local_irq_disable			# preempt stop
	b	__ret_from_irq
FEXPORT(ret_from_irq)
	LONG_S	s0, TI_REGS($28)
FEXPORT(__ret_from_irq)
	LONG_L	t0, PT_STATUS(sp)		# returning to kernel mode?
	andi	t0, t0, KU_USER
	beqz	t0, resume_xen

resume_userspace:
	local_irq_disable		# make sure we dont miss an
	j	restore_all


FEXPORT(restore_all)			# restore full frame
#ifdef CONFIG_MIPS_MT_SMTC
/* Detect and execute deferred IPI "interrupts" */
	LONG_L	s0, TI_REGS($28)
	LONG_S	sp, TI_REGS($28)
	jal	deferred_smtc_ipi
	LONG_S	s0, TI_REGS($28)
#ifdef CONFIG_MIPS_MT_SMTC_IM_BACKSTOP
/* Re-arm any temporarily masked interrupts not explicitly "acked" */
	mfc0	v0, CP0_TCSTATUS
	ori	v1, v0, TCSTATUS_IXMT
	mtc0	v1, CP0_TCSTATUS
	andi	v0, TCSTATUS_IXMT
	_ehb
	mfc0	t0, CP0_TCCONTEXT
	DMT	9				# dmt t1
	jal	mips_ihb
	mfc0	t2, CP0_STATUS
	andi	t3, t0, 0xff00
	or	t2, t2, t3
	mtc0	t2, CP0_STATUS
	_ehb
	andi	t1, t1, VPECONTROL_TE
	beqz	t1, 1f
	EMT
1:
	mfc0	v1, CP0_TCSTATUS
	/* We set IXMT above, XOR should clear it here */
	xori	v1, v1, TCSTATUS_IXMT
	or	v1, v0, v1
	mtc0	v1, CP0_TCSTATUS
	_ehb
	xor	t0, t0, t3
	mtc0	t0, CP0_TCCONTEXT
#endif /* CONFIG_MIPS_MT_SMTC_IM_BACKSTOP */
#endif /* CONFIG_MIPS_MT_SMTC */
	.set	noat
	RESTORE_TEMP
	RESTORE_AT
	RESTORE_STATIC
FEXPORT(restore_partial)		# restore partial frame
#ifdef CONFIG_TRACE_IRQFLAGS
	SAVE_STATIC
	SAVE_AT
	SAVE_TEMP
	LONG_L	v0, PT_STATUS(sp)
#if defined(CONFIG_CPU_R3000) || defined(CONFIG_CPU_TX39XX)
	and	v0, ST0_IEP
#else
	and	v0, ST0_IE
#endif
	beqz	v0, 1f
	jal	trace_hardirqs_on
	b	2f
1:	jal	trace_hardirqs_off
2:
	RESTORE_TEMP
	RESTORE_AT
	RESTORE_STATIC
#endif
	RESTORE_SOME
	RESTORE_SP_AND_RET
	.set	at

	.align 16
ENTRY(hypercall_table)
        .quad do_set_trap_table     /*  0 */
        .quad do_ni_hypercall
        .quad do_ni_hypercall
        .quad do_ni_hypercall
        .quad do_set_callbacks
        .quad do_ni_hypercall       /* 5 */
        .quad do_sched_op_compat
        .quad do_ni_hypercall
        .quad do_ni_hypercall
        .quad do_ni_hypercall
        .quad do_ni_hypercall       /* 10 */
        .quad do_ni_hypercall
        .quad do_memory_op
        .quad do_multicall
        .quad do_ni_hypercall
        .quad do_set_timer_op       /* 15 */
        .quad do_ni_hypercall
        .quad do_xen_version
        .quad do_console_io
        .quad do_ni_hypercall
        .quad do_grant_table_op     /* 20 */
        .quad do_vm_assist
        .quad do_ni_hypercall
        .quad do_ni_hypercall
        .quad do_vcpu_op
        .quad do_ni_hypercall       /* 25 */
        .quad do_ni_hypercall
        .quad do_xsm_op
        .quad do_nmi_op
        .quad do_sched_op
        .quad do_ni_hypercall       /* 30 */
        .quad do_ni_hypercall
        .quad do_event_channel_op
        .quad do_ni_hypercall
        .quad do_ni_hypercall
        .quad do_sysctl       /* 35 */
        .quad do_domctl
        .quad do_ni_hypercall
        .quad do_ni_hypercall
		.quad do_domctl2
        .quad do_destroy_domain    /* 40 */
	.align 16

ENTRY(hypercall_args_table)
        .byte 1 /* do_set_trap_table    */  /*  0 */
        .byte 4 /* do_mmu_update        */
        .byte 2 /* do_set_gdt           */
        .byte 2 /* do_stack_switch      */
        .byte 2 /* do_set_callbacks     */
        .byte 1 /* do_fpu_taskswitch    */  /*  5 */
        .byte 2 /* do_sched_op_compat   */
        .byte 1 /* do_platform_op       */
        .byte 2 /* do_set_debugreg      */
        .byte 1 /* do_get_debugreg      */
        .byte 4 /* do_update_descriptor */  /* 10 */
        .byte 0 /* do_ni_hypercall      */
        .byte 2 /* do_memory_op         */
        .byte 2 /* do_multicall         */
        .byte 4 /* do_update_va_mapping */
        .byte 2 /* do_set_timer_op      */  /* 15 */
        .byte 1 /* do_event_channel_op_compat */
        .byte 2 /* do_xen_version       */
        .byte 3 /* do_console_io        */
        .byte 1 /* do_physdev_op_compat */
        .byte 3 /* do_grant_table_op    */  /* 20 */
        .byte 2 /* do_vm_assist         */
        .byte 5 /* do_update_va_mapping_otherdomain */
        .byte 0 /* do_iret              */
        .byte 3 /* do_vcpu_op           */
        .byte 0 /* do_ni_hypercall      */  /* 25 */
        .byte 4 /* do_mmuext_op         */
        .byte 1 /* do_xsm_op            */
        .byte 2 /* do_nmi_op            */
        .byte 2 /* do_sched_op          */
        .byte 2 /* do_callback_op       */  /* 30 */
        .byte 2 /* do_xenoprof_op       */
        .byte 2 /* do_event_channel_op  */
        .byte 2 /* do_physdev_op        */
        .byte 2 /* do_hvm_op            */
        .byte 1 /* do_sysctl            */  /* 35 */
        .byte 1 /* do_domctl            */
        .byte 2 /* do_kexec_op          */
        .byte 1 /* do_tmem_op           */
		.byte 1 /* do_domctl2           */
        .byte 0 /* do_destroy_domain    */  /* 40 */
