#include <xen/config.h>
#include <xen/types.h>
#include <xen/smp.h>

#include <asm/asm.h>
#include <asm/regdef.h>
#include <asm/mipsregs.h>
#include <asm/cache.h>
#include <asm/addrspace.h>

unsigned int icache_linesz = 32;
unsigned int icache_lines __attribute__((unused));

void flush_icache_range(unsigned long start, unsigned long end)
{
	struct vaddr_pair args;

	args.start = start; args.end = end;

	/* 
	 * TODO: don't even send ipi to non-zero thread ids. This may 
	 * require some changes to smp_call_function interface, for 
	 * now just avoid redundant cache ops
	 */
	on_each_cpu(flush_icache_range_ipi, &args, 1, 1);
}

void local_flush_icache_all(void)
{
	int i = 0;
	int icache_lines, icache_sets, icache_assoc, icache_line_size;
	uint32_t config1;
	unsigned long base = (unsigned long)CKSEG0;

	config1 = read_c0_config1();
	icache_sets = 1 << (((config1 >> 22) & 0x7) + 6);
	icache_assoc = ((config1 >> 16) & 0x7) + 1;
	icache_line_size = 1 << (((config1 >> 19) & 0x7) + 1);

	icache_lines = icache_sets * icache_assoc;

	for (i = 0; i < icache_lines; i++) {
		cacheop(Index_Invalidate_I, base);
		base += icache_line_size;
	}
}
