#include <xen/config.h>
#include <xen/smp.h>

#include <asm/asm.h>
#include <asm/page.h>
#include <asm/bug.h>
#include <asm/system.h>

/* These are the functions hooked by the memory management function pointers */
void clear_page(void *page)
{
	__asm__ __volatile__(
		".set push                  \n"
		".set noreorder             \n"
		".set noat                  \n"
		".set mips4                 \n"

		STR(LONG_ADDIU) "\t$1, $0, 1		\n"
		STR(LONG_SLL)"\t$1, %1		\n"
		/* store the address of last cacheline in $1 */
		STR(LONG_ADDU) "\t$1, %0, $1  \n"  

		/* pref with prep_for_store and zero the cacheline */
		"1:   pref      30,  0(%0)  \n"

		"     sd        $0,  0(%0)  \n"  
		"     sd        $0,  8(%0)  \n"
		"     sd        $0, 16(%0)  \n"
		"     sd        $0, 24(%0)  \n"

		/* loop till the last cacheline */
		STR(LONG_ADDIU)"\t%0, %0, 32  \n"  
		"     bne       %0, $1, 1b  \n"
		"     nop                   \n"

		".set pop                   \n"
		:
		:"r" (page), "I" (PAGE_SHIFT)
		:"$1","memory");
}

void copy_page(void *to, void *from)
{
	__asm__ __volatile__(
		".set push                  \n"
		".set noreorder             \n"
		".set noat                  \n"
		".set mips64                 \n"

		STR(LONG_ADDIU) "\t$1, $0, 1		\n"
		STR(PTR_SLL) "\t $1, %2		\n"
		/* store the address of last cacheline in $1 */
		STR(LONG_ADDU)"\t$1, %1, $1  \n"  

		/* pref with prep_for_store the current cacheline */
		/* the stores should merge into this pref cacheline */
		"1:   pref      30,   0(%0)  \n"

		/* pref the next cacheline "from" */
		"     pref      0,    32(%1)  \n"

		/* copy the cacheline */
#ifdef CONFIG_32BIT
		"2:   lw        $8,   0(%1)  \n"
		"     lw        $9,   4(%1)  \n"
		"     lw        $10,  8(%1)  \n"
		"     lw        $11,  12(%1)  \n"
		"     sw        $8,   0(%0)  \n"
		"     sw        $9,   4(%0)  \n"
		"     sw        $10,  8(%0)  \n"
		"     sw        $11,  12(%0)  \n"
		"     lw        $8,   16(%1)  \n"
		"     lw        $9,   20(%1)  \n"
		"     lw        $10,  24(%1)  \n"
		"     lw        $11,  28(%1)  \n"
		"     sw        $8,   16(%0)  \n"
		"     sw        $9,   20(%0)  \n"
		"     sw        $10,  24(%0)  \n"
		"     sw        $11,  28(%0)  \n"
#else
		"2:   ld        $8,   0(%1)  \n"
		"     ld        $9,   8(%1)  \n"
		"     ld        $10,  16(%1)  \n"
		"     ld        $11,  24(%1)  \n"
		"     sd        $8,   0(%0)  \n"
		"     sd        $9,   8(%0)  \n"
		"     sd        $10,  16(%0)  \n"
		"     sd        $11,  24(%0)  \n"
#endif

		/* loop till the last cacheline */		       
		STR(LONG_ADDIU)"\t%1, %1, 32  \n"
		STR(LONG_ADDIU)"\t%0, %0, 32  \n"  
		"     bne       %1, $1, 1b  \n"
		"     nop                   \n"

		".set pop                   \n"

		:
		:"r" (to), "r"(from), "I" (PAGE_SHIFT)
		:"$1","$8", "$9", "$10", "$11", "memory");
}
