#include <xen/config.h>
#include <asm/asm.h>
#include <asm/asm-offsets.h>
#include <asm/regdef.h>
#include <asm/mipsregs.h>
#include <asm/cpu.h>
#include <asm/stackframe.h>
#include <asm/asmmacro.h>
		
NESTED(tail_jump, 16, sp)
	move	sp, a1
	move	gp, a2
	jr	    a0
	nop	
END(tail_jump)
		
	.macro 	__start_secondary
	.set push
	/*
	 * Dummy write for A0 bug in MMU
	*/
	li      t0, MMU_SETUP
	mtcr    zero, t0
	ehb
	SET_MIPS64
	setup_c0_status_sec		
	mfc0 t0, $15, 1
	andi t0, 0x1f
	PTR_LA	t1, secondary_stack
	li   t2, _THREAD_SIZE
	srl  t2, 2
	mul  t3, t2, t0
	nop
	nop
	nop
	nop
	nop
	nop
	PTR_ADDU  gp, t1, t3
	PTR_ADDU  sp, gp, t2
	PTR_ADDI  sp, sp, -64
	PTR_LA t0, start_secondary
	jr t0
	nop
	.set pop
    .endm

NESTED(boot_siblings, 0, sp)
	.set noreorder
	LONG_S  sp, xensp($0)
	SAVE_ALL
	sync
EXPORT(__boot_siblings)
	li      t1, (NR_THREADS - 1) /* wake up the 3 other threads */
	li      t0, ((CPU_BLOCKID_MAP << 8) | BLKID_MAP_THREADMODE)
	mfcr    t2, t0
	or	    t2, t2, t1
	mtcr	t2, t0
	mfc0    t0, CP0_EBASE, EBASE_SELECT_CPUINFO
	andi    t0, (CORE_NUMBER_MASK | THREAD_NUMBER_MASK)
	beqz    t0, 2f
	nop
1:
    __start_secondary
2:
	LONG_L   sp, xensp($0)
	PTR_SUBU sp, PT_SIZE
	RESTORE_ALL
	jr   ra
END(boot_siblings)

.macro __config_lsu
        .set push
        .set noreorder
        li      t0, LSU_DEFEATURE
        mfcr    t1, t0

        lui     t2, 0x4080  # Enable Unaligned Access, L2HPE
        or      t1, t1, t2

        li      t2, ~0xe    # S1RCM
        and     t1, t1, t2

        mtcr    t1, t0

        li      t0, SCHED_DEFEATURE
        lui     t1, 0x0100 # Experimental: Disable BRU accepting ALU ops
        mtcr    t1, t0

       .set pop
       .endm

EXPORT(reset_entry)
	mfc0    t0, CP0_EBASE, 1
	srl     t0, t0 , 2
	and     t0, t0 , 0x7 # t0 contains the core number
	li      t1, 0x1
	sll     t0, t1, t0
	nor     t0, t0, zero
	dla     t2, CPU_MMIO_OFFSET(SYS)
	lw      t1, (SYS_CPUNONCOHERENTMODE_REG << 2)(t2)
	and     t1, t1, t0
	sw      t1, (SYS_CPUNONCOHERENTMODE_REG << 2)(t2)

	/* read back to ensure complete */
	lw      t1, (SYS_CPUNONCOHERENTMODE_REG << 2)(t2)
	sync

	/* Configure LSU on Non-0 Cores. */
	__config_lsu
	
	dla 	k0,	__boot_siblings
	jr	    k0
	nop
EXPORT(reset_entry_end)
