#include <xen/config.h>
#include <xen/types.h>
#include <xen/time.h>
#include <xen/seqlock.h>
#include <xen/sched.h>
#include <asm/ptrace.h>
#include <asm/netl/iomap.h>
#include <asm/netl/pic.h>

#define PIC_TIMER_MAX_VALUE 0xffffffffffffffffUL

#define get_itc() (PIC_TIMER_MAX_VALUE - pic_read_timer(6))

void update_domain_wallclock_time(struct domain *d)
{

}

seqlock_t xtime_lock = __SEQLOCK_UNLOCKED(xtime_lock);
static s_time_t stime_irq = 0x0;
static unsigned long itc_at_irq;
// static unsigned long itc_scale __read_mostly;
static unsigned long itc_scale;
static unsigned long pic_freq = 133000000; /* 133 Mhz */

u64 cycle_to_ns(u64 cycle)
{
    return (cycle * itc_scale) >> 32;
}

static inline u64 get_time_delta(void)
{
    s64      delta_itc;
    u64      cur_itc;
    
    cur_itc = get_itc();

    delta_itc = (s64)(cur_itc - itc_at_irq);

    /* 
	 * Ensure that the returned system time is monotonically increasing. 
	 */
    if ( unlikely(delta_itc < 0) ) 
		delta_itc = 0;

    return cycle_to_ns(delta_itc);
}


/*
 * Arch-specific API
 */
s_time_t get_s_time(void)
{
    s_time_t now;
    unsigned long seq;

    do {
		seq = read_seqbegin(&xtime_lock);
		now = stime_irq + get_time_delta();
    } while (unlikely(read_seqretry(&xtime_lock, seq)));

    return now;
}

void update_vcpu_system_time(struct vcpu *v)
{

}


void domain_set_time_offset(struct domain *d, int32_t time_offset_seconds)
{
    d->time_offset_seconds = time_offset_seconds;
}


extern void send_guest_vcpu_virq(struct vcpu *v, int virq);

void send_timer_event(struct vcpu *v)
{
    send_guest_vcpu_virq(v, VIRQ_TIMER);
}


struct tm wallclock_time(void)
{
	return gmtime(0);
}

/*
 * FIXME: the following was copied from ia 64 port. Massge
 *        it suit our needs
 */
int reprogram_timer(s_time_t timeout)
{
#if 0
    struct vcpu *v = current;
    s_time_t expire; 
    unsigned long seq, cur_itc, itm_next;

    if (!domain0_ready || timeout == 0) 
		return 1;

    do {    
        seq = read_seqbegin(&xtime_lock);
        if ((expire = timeout - NOW()) < 0)
            return 0;

        cur_itc = ia64_get_itc();
        itm_next = cur_itc + ns_to_cycle(expire);
    } while (unlikely(read_seqretry(&xtime_lock, seq))); 

    local_cpu_data->itm_next = itm_next;

    vcpu_set_next_timer(v);
#endif

	return 1;
}

DEFINE_PER_CPU(unsigned long, nr_timer_int);

/*
 * FIXME: Compute cycles_per_jiffy during run time
 */
#define CYCLES_PER_JIFFY 16000000

void xen_timer_interrupt(int irq, void *info, struct cpu_user_regs *regs)
{
	unsigned int count, compare;
	unsigned long old_itc;

	count = read_c0_count();
	compare = read_c0_compare();
	
	if ((count >= compare) || ((compare - count) > CYCLES_PER_JIFFY)) {
		write_seqlock(&xtime_lock);

		/* Updates system time (nanoseconds since boot). */
		old_itc = itc_at_irq;
		itc_at_irq = get_itc();
		stime_irq += cycle_to_ns(itc_at_irq - old_itc);		
		write_sequnlock(&xtime_lock);

		per_cpu(nr_timer_int, smp_processor_id()) += 1;
		write_c0_compare(count + CYCLES_PER_JIFFY);
	}
}

int init_xen_time(void)
{
	int retval = 0;

	write_c0_compare(read_c0_count() + CYCLES_PER_JIFFY);

	retval = request_irq_vector(TIMER_VECTOR, xen_timer_interrupt, IRQ_PER_CPU, "timer", NULL);
	if ((retval != 0) && (retval != -EBUSY))
		goto exit;

    itc_scale  = 1000000000UL << 32 ;
    itc_scale /= pic_freq;

   /* System time ticks from zero. */
    stime_irq = (s_time_t)0;
    itc_at_irq = get_itc();

    printk("Time init:\n");
    printk(".... System Time: %ld nanoseconds\n", NOW());
    printk(".... scale: %16lX\n", itc_scale);

exit:
	return retval;
}
