#include <xen/config.h>
#include <xen/init.h>
#include <xen/sched.h>
#include <xen/lib.h>
#include <xen/errno.h>
#include <xen/mm.h>
#include <xen/console.h>
#include <xen/shutdown.h>
#include <asm/regs.h>
#include <xen/delay.h>
#include <xen/event.h>

#include <asm/thread_info.h>

#include <xen/spinlock.h>
#include <xen/irq.h>
#include <xen/perfc.h>
#include <xen/smp.h>
#include <xen/softirq.h>
#include <xen/domain_page.h>
#include <xen/symbols.h>
#include <xen/iocap.h>
#include <xen/nmi.h>
#include <xen/version.h>
#include <xen/kexec.h>
#include <xen/trace.h>
#include <xen/paging.h>

#include <asm/system.h>

#include <asm/io.h>
#include <asm/atomic.h>
#include <asm/desc.h>
#include <asm/smp.h>
#include <asm/flushtlb.h>
#include <asm/uaccess.h>
#include <asm/debugger.h>
#include <asm/traps.h>
#include <asm/ptrace.h>
#include <asm/cache.h>
#include <asm/domain.h>

#include <public/callback.h>

#define PAGE_4K (4 << 10) /* move this to header file if needed in multiple places */

/*
 * Implement the hypercall set_trap_table()
 */

unsigned long ebase = 0x0;
unsigned long exception_handlers[32];

void __init set_handler(unsigned long offset, void *addr, unsigned long size)
{
	memcpy((void *)(ebase + offset), addr, size);
	flush_icache_range(ebase + offset, ebase + offset + size);
}



unsigned long do_set_trap_table(XEN_GUEST_HANDLE(const_trap_info_t) traps)
{
#if 0
    struct trap_info cur;
    struct vcpu *curr = current;
    struct trap_info *dst = curr->arch.guest_context.trap_ctxt;
    long rc = 0;
#endif

	return 0;
	
}

void vcpu_show_execution_state(struct vcpu *v)
{

}

/*
 * trap_bounce structure is pointed to in the 'struct vcpu' structure
 * Who allocates the structure for that ? Guest domains or Xen ?
 */

#if 0
void propagate_page_fault(unsigned long addr, u16 error_code)
{
    struct trap_info *ti;
    struct vcpu *v = current;
    struct trap_bounce *tb = &v->arch.trap_bounce;

    v->arch.guest_context.ctrlreg[2] = addr;
    arch_set_cr2(v, addr);

    /* Re-set error_code.user flag appropriately for the guest. */
    error_code &= ~PFEC_user_mode;
    if ( !guest_kernel_mode(v, guest_cpu_user_regs()) )
        error_code |= PFEC_user_mode;

    trace_pv_page_fault(addr, error_code);

    ti = &v->arch.guest_context.trap_ctxt[TRAP_page_fault];
    tb->flags = TBF_EXCEPTION | TBF_EXCEPTION_ERRCODE;
    tb->error_code = error_code;
    tb->cs         = ti->cs;
    tb->eip        = ti->address;
    if ( TI_GET_IF(ti) )
        tb->flags |= TBF_INTERRUPT;
    if (unlikely(null_trap_bounce(v, tb))) {
        printk("d%d:v%d: unhandled page fault (ec=%04X)\n",
               v->domain->domain_id, v->vcpu_id, error_code);
        show_page_walk(addr);
    }

    if ( unlikely(error_code & PFEC_reserved_bit) )
        reserved_bit_page_fault(addr, guest_cpu_user_regs());
}
#endif

/*
 * Page-fault Handling Mechanism
 *
 * What should be the functionality of page-fault handling ? It is
 * just a page-fault handler. Right ? Since page fault has its own
 * interrupt vectors, tlbl, tlbs, tlbm
 *
 * The following is a generic routine and has multiple entry points,
 * the entry points being the exception vectors/trap handlers for
 * the interrupts tlbl, tlbs, tlbm. So, we need to have the 
 * distinguishing values as one of the arguments. In x86, the
 * error code is retrieved from a register I guess. In Xen, the
 * error code is recorded in one of the registers as part of
 * context-save ?
 * 
 */

asmlinkage void do_page_fault(struct cpu_user_regs *regs, int error, unsigned long address)
{
	/*
	 * In Linux, we just send a signal for illegal access, but in
	 * Xen, we need to bounce this to the appropriate guest domain
	 * The case for which hypervisor might take page fault is for
	 * controlled cases when we specify the fixup mechanism in the
	 * fixup section
	 */
	
	/*
	 * we do not fixup page faults in interrupt context. What does
	 * that mean ? What is an external-mode guest ?
	 */

	if (unlikely(!guest_mode(regs))) {
#if 0
		if (spurious_page_fault(address, error))
			return;
		if (likely((fixup = search_exception_table(regs->eip)) != 0)) {

		}
#endif
	}

	/*
	 * we will have to pass more than address and error to guest 
	 * domains. However, regs could be from the kernel stack and
	 * guest domains will not be able to acess that address space
	 * For now retain regs. If it fails, try carving some shared
	 * space and pass the information there
	 */
	// propagate_page_fault(regs, error, address);
}

#define NETL_MIPS_EBASE 0xffffffff80000000
#define NR_EXC_CODE 32

extern char except_vec_tlbrefill;
extern char except_vec_xtlbrefill;
extern char except_vec_generic;
extern char except_vec_cacheerror;
extern char except_vec_vecint;

extern asmlinkage void handle_int(void);
extern asmlinkage void handle_reserved(void);

void trap_init(void)
{
	int i;

	ebase = NETL_MIPS_EBASE + (processor_id() * PAGE_4K);

	exception_handlers[0] = (unsigned long)&handle_int;
	for (i = 1; i < NR_EXC_CODE; i++)
		exception_handlers[i] = (unsigned long)&handle_reserved;


	memcpy((void *)ebase, &except_vec_tlbrefill, 0x80);
	memcpy((void *)(ebase + 0x80),  &except_vec_xtlbrefill, 0x80);
	memcpy((void *)(ebase + 0x180), &except_vec_generic, 0x80);
	memcpy((void *)(0xffffffffa0000000 + 0x100), &except_vec_cacheerror, 0x80);
	memcpy((void *)(ebase + 0x200), &except_vec_vecint, 0x80);

	/* 
	 * set up the ebase register
	 */
	write_c0_ebase((uint32_t)(ebase & 0x3ffff000));

	/* 
	 * Flush I-Cache 
	 */
	local_flush_icache_all();
#if 0
	if (cpu_has_divec) {
		if (cpu_has_mipsmt) {
			unsigned int vpflags = dvpe();
			set_c0_cause(CAUSEF_IV);
			evpe(vpflags);
		} else
			set_c0_cause(CAUSEF_IV);
	}
	else {
		clear_c0_cause(CAUSEF_IV);
	}
#endif
}

static long register_guest_callback(struct callback_register *reg)
{
    switch (reg->type)
    {
    case CALLBACKTYPE_dom0_getfreepages:
		if (current->domain->domain_id != 0)
			return -EPERM;
		get_free_pages = (void *)reg->address;
		break;
	default:
		break;
	}

	return 0;
}	

long do_set_callbacks(unsigned long event_selector, unsigned long event_address)
{
    struct callback_register event = {
        .type = event_selector,
        .address = event_address,
    };

    register_guest_callback(&event);

#ifdef DEBUG
	printk("get_free_pages = %p\n", get_free_pages);
#endif

    return 0;
}
