#ifndef __ASM_BOOTINFO_H
#define __ASM_BOOTINFO_H

#include <xen/types.h>

// #define PSB_INFO_VERSION 0x0001
#define MEMMAP_MAX_ENTRIES 32

/*
 * The following structure has three additional fields
 * than its counter part in the bootloader. Thanks to
 * damaged brains, bootloader passes four arguments
 * (including the following structure) when the 3 other
 * arguments could have been folded into this structure
 * and a single pointer passed to Xen/Linux
 */

struct bootinfo {
	uint64_t boot_level;
	uint64_t io_base;
	uint64_t output_device;
	uint64_t uart_print; /* do not use this */
	uint64_t led_output; /* " */
	uint64_t init;
	uint64_t exit;
	uint64_t warm_reset;
	uint64_t wakeup;
	uint64_t cpu_online_map;
	uint64_t master_reentry_sp;
	uint64_t master_reentry_gp;
	uint64_t master_reentry_fn;
	uint64_t slave_reentry_fn;
	uint64_t magic_dword;
	uint64_t uart_putchar;
	uint64_t size;
	uint64_t uart_getchar;
	uint64_t nmi_handler;
	uint64_t psb_version;
	uint64_t mac_addr;
	uint64_t cpu_frequency;
	uint64_t board_version;
	uint64_t malloc;
	uint64_t free;
	uint64_t global_shmem_addr;
	uint64_t global_shmem_size;
	uint64_t psb_os_cpu_map;
	uint64_t userapp_cpu_map;
	uint64_t wakeup_os; /* unused */
	uint64_t psb_memory_map;
	uint64_t board_major_version;
	uint64_t board_minor_version;
	uint64_t board_manf_revision;
	uint64_t board_serial_number;
	uint64_t paddr_map;
	uint64_t xlr_loaderip_config;
	uint64_t bldr_envp;
	uint64_t avail_memory_map;
};

/*
 * A memory map that's built upon what was determined
 * or specified on the command line.
 */
struct addrmap {
	int nr_map;
	struct addrmap_entry {
		uint64_t addr;  /* start of memory segment */
		uint64_t size;  /* size of memory segment */
		uint32_t type;  /* type of memory segment */
	} map[MEMMAP_MAX_ENTRIES + 1];
};

int read_bootinfo(struct bootinfo *bootinfo);

#define COMMAND_LINE_SIZE 256

extern struct addrmap *drammap_p, *paddrmap_p, *scratchmap_p;

struct typename {
	int type;
	char *name;
};

typedef enum
{
	RAM_MEMORY = 1,
	ROM_MEMORY,
	RESERVED_MEMORY,
	UNKNOWN_TYPE	
} dram_types;

extern void print_addrmap(struct addrmap *addrmap, struct typename *map, int size);
extern struct typename dram_descriptors[UNKNOWN_TYPE];

extern int g_argc;
extern char *g_argv[], *g_envp[];

extern struct bootinfo bootinfo;
extern unsigned long fw_arg0, fw_arg1, fw_arg2, fw_arg3;
extern void *fdt;

#endif
