/*
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright (C) 1997, 98, 99, 2000, 2003 Ralf Baechle
 * Copyright (C) 1999 Silicon Graphics, Inc.
 */
#ifndef _ASM_CACHE_H
#define _ASM_CACHE_H

#include <xen/config.h>
#include <xen/types.h>
#include <asm/asm.h>

#ifdef __KERNEL__
#include <kmalloc.h>
#endif

#define L1_CACHE_SHIFT		CONFIG_XEN_MIPS_L1_CACHE_SHIFT
#define L1_CACHE_BYTES		(1 << L1_CACHE_SHIFT)

#define SMP_CACHE_SHIFT		L1_CACHE_SHIFT
#define SMP_CACHE_BYTES		L1_CACHE_BYTES

#define ARCH_KMALLOC_MINALIGN   8

#define __read_mostly __attribute__((__section__(".data.read_mostly")))

/*
 * TODO: Move the following structure definitions to
 *       a header file
 */
struct vaddr_pair {
	unsigned long start;
	unsigned long end;
};

struct paddr_pair {
	phys_t start;
	phys_t end;
};

#ifdef CONFIG_RMI_XLP

static __inline__ void pipeline_flush(void)
{
	__asm__ __volatile__ (
		".set push         \n"
		".set arch=xlp     \n"
		"dla      $8, 1f    \n"
		"jr.hb   $8        \n"
		"nop               \n"
		"1: nop            \n"
		".set pop          \n"
		:
		:
		: "$8"
		);
}

#else /* !CONFIG_RMI_XLP */

static __inline__ void pipeline_flush(void)
{
	__asm__ __volatile__ (                                    
		".set push      \n"    
		".set noreorder \n"    
		"nop            \n"
		"nop            \n"
		"nop            \n"
		"nop            \n"
		"nop            \n"
		"nop            \n"
		"nop            \n"
		"nop            \n"
		"nop            \n"
		"nop            \n"
		".set pop       \n"  
	  );
}

#endif /* !CONFIG_RMI_XLP */

#define sync_istream          pipeline_flush
#define cacheop_hazard        pipeline_flush

extern unsigned int icache_linesz, icache_lines;

static __inline__ void cacheop(int op, unsigned long base)
{
	__asm__ __volatile__ (
		".set push \n"   
		".set noreorder \n"   
		".set mips4 \n"   
		"cache %0, 0(%1) \n"  
		".set pop \n"   
		: 
		: "i" (op), "r" (base)
		);
}

static __inline__ void cacheop_extable(int op, unsigned long base)
{
	__asm__ __volatile__ (
		".set push \n"
		".set noreorder \n"
		".set mips4 \n"
		"1:  cache %0, 0(%1) \n"
		"2:  .set pop \n"
		"    .section __ex_table,\"a\"\n"
		STR(PTR)"\t1b, 2b\n\t"
		"     .previous \n"
		: : "i" (op), "r" (base)
		);
}

#define Index_Invalidate_I 0x00
#define Hit_Invalidate_I 0x10

static inline void local_flush_icache_range(unsigned long start, unsigned long end)
{
	unsigned long addr;

	for (addr = (start & ~((unsigned long)(icache_linesz - 1))); addr < end; 
		addr += icache_linesz)
		cacheop_extable(Hit_Invalidate_I, addr);

	sync_istream();
}

static inline void flush_icache_range_ipi(void *info)
{
	struct vaddr_pair *args = info;

	local_flush_icache_range(args->start, args->end);
}

extern void flush_icache_range(unsigned long, unsigned long);
extern void local_flush_icache_all(void);

#endif /* _ASM_CACHE_H */
