/*
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright (C) 1994 by Waldorf Electronics
 * Copyright (C) 1995 - 2000, 01, 03 by Ralf Baechle
 * Copyright (C) 1999, 2000 Silicon Graphics, Inc.
 */
#ifndef _ASM_DELAY_H
#define _ASM_DELAY_H

#include <xen/config.h>
#include <xen/smp.h>
#include <asm/mipsregs.h>
#include <asm/cpu-info.h>

#if defined(CONFIG_RMI_PHOENIX) || defined(CONFIG_RMI_XLP)

static inline void __udelay(unsigned long usecs, unsigned long lpj)
{
	uint32_t initial_count,curr_count;
	unsigned long delta, elapsed;

	delta = usecs * (cpu_frequency / 1000000);

	initial_count = read_c0_count();

	while (1) {
		curr_count = read_c0_count();
		if (likely(curr_count > initial_count))
			elapsed = curr_count - initial_count;
		else if (curr_count < initial_count)
			elapsed = 0xffffffff - (initial_count - curr_count);
		else
			elapsed = 0xffffffff;			
		
		if ((uint32_t)elapsed > delta)
			return;
	}
}

#else /* CONFIG_RMI_PHOENIX */

static inline void __delay(unsigned long loops)
{

}


/*
 * Division by multiplication: you don't have to worry about
 * loss of precision.
 *
 * Use only for very small delays ( < 1 msec).  Should probably use a
 * lookup table, really, as the multiplications take much too long with
 * short delays.  This is a "reasonable" implementation, though (and the
 * first constant multiplications gets optimized away if the delay is
 * a constant)
 */

static inline void __udelay(unsigned long usecs, unsigned long lpj)
{
	__delay(usecs);
}

// #define __udelay_val cpu_data[raw_smp_processor_id()].udelay_val
#endif /* CONFIG_RMI_PHOENIX */

#define udelay(usecs) __udelay((usecs),0)

#if 0

/* make sure "usecs *= ..." in udelay do not overflow. */
#if HZ >= 1000
#define MAX_UDELAY_MS	1
#elif HZ <= 200
#define MAX_UDELAY_MS	5
#else
#define MAX_UDELAY_MS	(1000 / HZ)
#endif
#endif

#endif /* _ASM_DELAY_H */
