#ifndef __ASM_DOMAIN_H__
#define __ASM_DOMAIN_H__

#include <xen/config.h>
#include <xen/mm.h>
#include <xen/cache.h>
#include <asm/processor.h>
#include <linux/gfp.h>

struct trap_bounce {
    uint32_t      error_code;
    uint8_t       flags; /* TBF_ */
    unsigned long eip;
};

#define MAPHASH_ENTRIES 8
#define MAPHASH_HASHFN(pfn) ((pfn) & (MAPHASH_ENTRIES-1))
#define MAPHASHENT_NOTINUSE ((u16)~0U)

struct mapcache_vcpu {

};

#define MAPCACHE_ORDER   10
#define MAPCACHE_ENTRIES (1 << MAPCACHE_ORDER)

struct mapcache_domain {
};

void mapcache_domain_init(struct domain *);
void mapcache_vcpu_init(struct vcpu *);

/* x86/64: toggle guest between kernel and user modes. */
void toggle_guest_mode(struct vcpu *);

/*
 * Initialise a hypercall-transfer page. The 
 * given pointer must be mapped in Xen virtual 
 * address space (accesses are not validated 
 or checked).
 */
void hypercall_page_initialise(struct domain *d, void *);

/*          
 * shadow paging extension
 */

struct shadow_domain {
};

struct shadow_vcpu {
};

/*            
 * hardware assisted paging          
 */
struct hap_domain {
};


/*
 * common paging data structure           
 */

struct log_dirty_domain {
};

struct paging_domain {
};

struct paging_vcpu {
};

#define MAX_CPUID_INPUT 40
// typedef xen_domctl_cpuid_t cpuid_input_t;

struct p2m_domain;

/*
 * These regions keep track of the virtual memory mappings
 * obtained from the guest image (e.g code, data sections, etc).
 * They are used to create the bootstrap page tables and setup initial
 * tlb entries.
 */
#define MAX_MIPS_MAP_REGIONS 8

typedef struct arch_map_regions {
  unsigned long virt_start; 
  unsigned long nr_pages;
} arch_map_regions_t;

struct arch_domain
{
	/*
	 * @doc: mm represents the page table of the domain
	 *       itself, since guest domains run in mapped
	 *       space. Xen will have to populate this structutre
	 *       while loading guest domains. What about the
	 *       pte entry ? Does it capture physical/machine
	 *       address ? Since this page table is used by
	 *       Xen alone, pte(s) could represent the machine
	 *       address
	 */ 
    struct mm_struct mm;
	unsigned long virt_start;
	unsigned long virt_end;
	struct rangeset *ioport_caps;
	struct page_list_head relmem_list;
	unsigned int page_size;
	int nr_map_entries;
	arch_map_regions_t map_regions[MAX_MIPS_MAP_REGIONS];
	/* Used by continuable domain_relinquish_resources(). */
	enum {
		RELMEM_not_started,
		RELMEM_dom,
		RELMEM_xen,
		RELMEM_done,
	} relmem;
} __cacheline_aligned;


struct arch_vcpu
{
    /* Needs 16-byte aligment for FXSAVE/FXRSTOR. */
    struct vcpu_guest_context guest_context  __attribute__((__aligned__(16)));
	struct thread_info *thread_info;

    unsigned long      flags; /* TF_ */

    /*
     * the hook to the scheduler function
     * of each domain. each domain registers
     * this function during startup
     */
    void (*schedule_tail) (struct vcpu *);
    void (*ctxt_switch_from) (struct vcpu *);
    void (*ctxt_switch_to) (struct vcpu *);
    unsigned long syscall_callback;

    /*
     * might need:
     * a) trap-bounce structures
     * b) hypercall continuation
     * c) any other book-keeping needed by domains
     *    such as for migration
     */

    /*
     * Need?:
     * a) pointers to page tables in domains
     */

    /* 
     * guest_table holds a ref to the page, and also a
     * type-count unless shadow refcounts are in use 
     */
    pgd_t shadow_table[4];        /* (MFN) shadow(s) of guest */
    pgd_t monitor_table;          /* (MFN) hypervisor PT (for HVM) */
    unsigned long cr3;                  /* (MA) value to install in HW CR3 */
} __cacheline_aligned;

/* Continue the current hypercall via func(data) on specified cpu. */
int continue_hypercall_on_cpu(int cpu, long (*func)(void *data), void *data);
void vcpu_show_execution_state(struct vcpu *);
void vcpu_show_registers(const struct vcpu *);

void domain_cpuid(struct domain *d,
                  unsigned int  input,
                  unsigned int  sub_input,
                  unsigned int  *eax,
                  unsigned int  *ebx,
                  unsigned int  *ecx,
                  unsigned int  *edx);

int construct_dom0(
    struct domain *d,
//    unsigned long image_base,
    unsigned long image_start, unsigned long image_len,
    unsigned long initrd_start, unsigned long initrd_len,
    char *cmdline);

extern unsigned long dom0_memory_start, dom0_memory_size;
extern unsigned long dom0_initrd_start, dom0_initrd_size;

extern unsigned long (*get_free_pages)(gfp_t gfp_mask, unsigned int order);

#define DOMU_SSIDREF 1

extern long do_destroy_domain(void);

#endif /* __ASM_DOMAIN_H__ */
