#ifndef _ASM_RMI_MIPS_EXTS_H
#define _ASM_RMI_MIPS_EXTS_H

#ifndef __ASSEMBLY__

#include <xen/types.h>
#include <asm/netl/interrupt.h>

#define DMFC0_AT_EIRR 0x40214806
#define DMFC0_AT_EIMR 0x40214807
#define DMTC0_AT_EIRR 0x40a14806
#define DMTC0_AT_EIMR 0x40a14807

/* functions to write to and read from the extended
 * cp0 registers.
 * EIRR : Extended Interrupt Request Register
 *        cp0 register 9 sel 6
 *        bits 0...7 are same as cause register 8...15
 * EIMR : Extended Interrupt Mask Register
 *        cp0 register 9 sel 7
 *        bits 0...7 are same as status register 8...15
 */

static inline __u64 read_64bit_cp0_eirr(void)
{
	__u32 high, low;

	__asm__ __volatile__ (
		".set push\n"
		".set noreorder\n"
		".set noat\n"
		".set mips4\n"
		".word 0x40214806  \n\t"
		"nop               \n\t"
		"dsra32 %0, $1, 0  \n\t"
		"sll    %1, $1, 0  \n\t"
		".set pop\n"
		: "=r" (high), "=r" (low)
		);

	return (((__u64)high) << 32) | low;
}

static inline __u64 read_64bit_cp0_eimr(void)
{
	__u32 high, low;

	__asm__ __volatile__ (
		".set push\n"
		".set noreorder\n"
		".set noat\n"
		".set mips4\n"
		".word 0x40214807  \n\t"
		"nop               \n\t"
		"dsra32 %0, $1, 0  \n\t"
		"sll    %1, $1, 0  \n\t"
		".set pop\n"
		: "=r" (high), "=r" (low)
		);

	return (((__u64)high) << 32) | low;
}

static inline void write_64bit_cp0_eirr(__u64 value)
{
	__u32 low, high;

	high = value >> 32;
	low  = value & 0xffffffff;

	__asm__ __volatile__ (
		".set push\n"
		".set noreorder\n"
		".set noat\n"
		".set mips4\n\t"
		"dsll32 $2, %1, 0  \n\t"
		"dsll32 $1, %0, 0  \n\t"
		"dsrl32 $2, $2, 0  \n\t"
		"or     $1, $1, $2 \n\t"
		".word  0x40a14806 \n\t"
		"nop               \n\t"
		".set pop\n"
		:
		: "r" (high), "r" (low)
		: "$1", "$2"
		);
}

static inline void write_64bit_cp0_eimr(__u64 value)
{
	__u32 low, high;

	high = value >> 32;
	low  = value & 0xffffffff;

	__asm__ __volatile__ (
		".set push\n"
		".set noreorder\n"
		".set noat\n"
		".set mips4\n\t"

		"dsll32 $2, %1, 0  \n\t"
		"dsll32 $1, %0, 0  \n\t"
		"dsrl32 $2, $2, 0  \n\t"
		"or     $1, $1, $2 \n\t"
		".word  0x40a14807 \n\t"
		"nop               \n\t"

		".set pop\n"

		:
		: "r" (high), "r" (low)
		: "$1", "$2"
		);
}

static __inline__ int ldadd_w(unsigned int value, volatile int *addr)
{
	unsigned long res;
	__asm__ __volatile__(
		".set push\n"
		".set noreorder\n"
		"move $8, %2\n"
		"move $9, %3\n"
		".word 0x71280010\n" 		/* "ldaddw %2, %3\n" */
		"move %0, $8\n"
		".set pop\n"
		:"=r"(res), "+m"(*addr)
		: "r" (value), "r"((unsigned long)addr)
		: "$8", "$9"
		);
	return res;
}

static __inline__ void ldadd_w_no_read(int value, volatile int *addr)
{
	unsigned long res;
	__asm__ __volatile__(
		".set push\n"
		".set noreorder\n"
		"move $8, %2\n"
		"move $9, %3\n"
		".word 0x71280010\n" /* "ldaddw $8, $9\n" */
		//"move %0, $8\n"
		".set pop\n"
		:"=r"(res), "+m"(*addr)
		: "r" (value), "r"((unsigned long)addr)
		: "$8", "$9"
		);
}

static __inline__ 
unsigned int ldadd_wu(unsigned int value, volatile unsigned int *addr)
{
	unsigned long res;
	__asm__ __volatile__(
		".set push\n"
		".set noreorder\n"
		"move $8, %2\n"
		"move $9, %3\n"
		//"ldaddwu $8, $9\n"
		".word 0x71280011\n"
		"move %0, $8\n"
		".set pop\n"
		:"=r"(res), "+m"(*addr)
		: "r"(value), "r"((unsigned long)addr)
		: "$8", "$9"
		);
	return res;
}

static __inline__ 
void ldadd_wu_no_read(unsigned int value, volatile unsigned int *addr)
{
	unsigned long res;
	__asm__ __volatile__(
		".set push\n"
		".set noreorder\n"
		"move $8, %2\n"
		"move $9, %3\n"
		//"ldaddwu $8, $9\n"
		".word 0x71280011\n"
		//"move %0, $8\n"
		".set pop\n"
		:"=r"(res), "+m"(*addr)
		: "r"(value), "r"((unsigned long)addr)
		: "$8", "$9"
		);
}

/*
 * FIXME: core_id calculation looks wrong, different from
 *        the description of ebase register, sel 1
 */
static inline int core_id(void)
{
	int __id;

	__asm__ __volatile__ (
		".set push\n"
		".set noreorder\n"
		".set mips32\n"
		"mfc0 $8, $15, 1\n"
		"andi $8, $8, 0x3ff\n"
		"srl %0, $8, 1\n"
		".set pop\n"
		: "=r" (__id) : : "$8"
		);

	return __id;
}

static inline int thread_id(void)
{
	int __id;
	
	__asm__ __volatile__ (                                         
		".set push\n"
		".set noreorder\n"
		".set mips32\n"
		"mfc0 $8, $15, 1\n"
		"andi %0, $8, 0x3\n"
		".set pop\n"
		: "=r" (__id) : : "$8"
		);

	return __id;
}

static __inline__ int processor_id(void)
{
	int cpu;
	
	__asm__ __volatile__ (
		".set push\n"
		".set noreorder\n"
		".set mips32\n"
		"mfc0 $8, $15, 1\n"
		"andi %0, $8, 0x3ff\n"
		".set pop\n"
		: "=&r"(cpu) : : "$8"
		);

	return cpu;
}

#define phoenix_cpu_to_thrid(cpu) (phys_proc_id[(cpu)] >> 2)
#define phoenix_cpu_to_cpuid(cpu) (phys_proc_id[(cpu)] & 0x3)

#define IFU_BLOCK      0
#define ICU_BLOCK      1
#define IEU_BLOCK      2
#define LSU_BLOCK      3
#define MMU_BLOCK      4
#define PRF_BLOCK      5

#define LSU_CERRLOG_REGID    9

static __inline__ unsigned int read_control_reg_w(int block, int reg)
{
	unsigned int __res;
	
	__asm__ __volatile__(
		".set\tpush\n\t"
		".set\tnoreorder\n\t"
		"move $9, %1\n"
		".word 0x71280018\n" /* mfcr\t$8, $9\n\t */
		"move %0, $8\n"
		".set\tpop"
		: "=r" (__res) : "r"((block<<8)|reg)
		: "$8", "$9"
		);
	return __res;
}

static __inline__ void write_control_reg_w(int block, int reg, unsigned int value)
{
	__asm__ __volatile__(
		".set\tpush\n\t"
		".set\tnoreorder\n\t"
		"move $8, %0\n"
		"move $9, %1\n"
		".word 0x71280019\n" /* mtcr\t$8, $9\n\t */
		".set\tpop"
		:
		: "r" (value), "r"((block<<8)|reg)
		: "$8", "$9"
		);
}

static __inline__ unsigned long long read_control_reg_dw(int block, int reg)
{
	unsigned int high, low;

	__asm__ __volatile__(
		".set\tmips64\n\t"
		"move    $9, %2\n"
		".word   0x71280018\n"    /* "mfcr $8, $9\n" */
		"dsrl32  %0, $8, 0\n\t"
		"dsll32  $8, $8, 0\n\t"
		"dsrl32  %1, $8, 0\n\t"
		".set mips0"
		: "=r" (high), "=r"(low)
		: "r"((block << 8) | reg)
		: "$8", "$9"
		);

	return ((((unsigned long long)high) << 32) | low);
}

static __inline__ void write_control_reg_dw(int block, int reg,unsigned long long value)
{
	__u32 low, high;
	high = value >> 32;
	low = value & 0xffffffff;

	__asm__ __volatile__(
		".set push\n"
		".set noreorder\n"
		".set mips4\n\t"
		/* Set up "rs" */
		"move $9, %0\n"
		/* Store 64 bit value in "rt" */
		"dsll32 $10, %1, 0  \n\t"
		"dsll32 $8, %2, 0  \n\t"
		"dsrl32 $8, $8, 0  \n\t"
		"or     $8, $10, $8 \n\t"
		".word 0x71280019\n" /* mtcr $8, $9 */
		".set pop\n"
		:  /* No outputs */
		: "r"((block << 8) | reg), "r" (high), "r" (low)
		: "$8", "$9", "$10"
		);
}

#endif

#ifdef CONFIG_CPU_XLP
#define SET_MIPS64 .set mips64r2
#else
#define SET_MIPS64 .set mips64
#endif

#endif /* _ASM_RMI_MIPS_EXTS_H */
