#ifndef _ASM_NETL_IOMAP_H
#define _ASM_NETL_IOMAP_H

#if !defined(CONFIG_RMI_XLP)
#define MMIO_BASEADDR 0xffffffffbef00000ULL
#define PHOENIX_IO_DDR2_CHN0_OFFSET       0x01000
#define PHOENIX_IO_DDR2_CHN1_OFFSET       0x02000
#define PHOENIX_IO_DDR2_CHN2_OFFSET       0x03000
#define PHOENIX_IO_DDR2_CHN3_OFFSET       0x04000
#define PHOENIX_IO_PIC_OFFSET             0x08000
#define PHOENIX_IO_UART_0_OFFSET          0x14000
#define PHOENIX_IO_UART_1_OFFSET          0x15100

#else
#define MMIO_BASEADDR 0xffffffffb8000000ULL
#define PHOENIX_IO_DDR2_CHN0_OFFSET       0x14000
#define PHOENIX_IO_DDR2_CHN1_OFFSET       0x15000
#define PHOENIX_IO_DDR2_CHN2_OFFSET       0x16000
#define PHOENIX_IO_DDR2_CHN3_OFFSET       0x17000
#define PHOENIX_IO_PIC_OFFSET             0x04000
#define PHOENIX_IO_UART_0_OFFSET          0x30100
#define PHOENIX_IO_UART_1_OFFSET          0x31100
#endif /* CONFIG_RMI_XLP */

#define PHOENIX_IO_SIZE                   0x1000

#define PHOENIX_IO_BRIDGE_OFFSET          0x00000


#define PHOENIX_IO_RLD2_CHN0_OFFSET       0x05000
#define PHOENIX_IO_RLD2_CHN1_OFFSET       0x06000

#define PHOENIX_IO_SRAM_OFFSET            0x07000

#define PHOENIX_IO_PCIX_OFFSET            0x09000
#define PHOENIX_IO_HT_OFFSET              0x0A000

#define PHOENIX_IO_SECURITY_OFFSET        0x0B000

#define PHOENIX_IO_GMAC_0_OFFSET          0x0C000
#define PHOENIX_IO_GMAC_1_OFFSET          0x0D000
#define PHOENIX_IO_GMAC_2_OFFSET          0x0E000
#define PHOENIX_IO_GMAC_3_OFFSET          0x0F000

#ifdef XLS
#define PHOENIX_IO_GMAC_4_OFFSET          0x20000
#define PHOENIX_IO_GMAC_5_OFFSET          0x21000
#define PHOENIX_IO_GMAC_6_OFFSET          0x22000
#define PHOENIX_IO_GMAC_7_OFFSET          0x23000

#define PHOENIX_IO_PCIE_0_OFFSET          0x1E000
#define PHOENIX_IO_PCIE_1_OFFSET          0x1F000

#define PHOENIX_IO_USB_0_OFFSET           0x24000
#define PHOENIX_IO_USB_1_OFFSET           0x25000

#define PHOENIX_IO_COMP_OFFSET            0x1D000

#endif /* XLS */

#define PHOENIX_IO_SPI4_0_OFFSET          0x10000
#define PHOENIX_IO_XGMAC_0_OFFSET         0x11000
#define PHOENIX_IO_SPI4_1_OFFSET          0x12000
#define PHOENIX_IO_XGMAC_1_OFFSET         0x13000

#define PHOENIX_IO_I2C_0_OFFSET           0x16000
#define PHOENIX_IO_I2C_1_OFFSET           0x17000

#define PHOENIX_IO_GPIO_OFFSET            0x18000

#define PHOENIX_IO_FLASH_OFFSET           0x19000

#define PHOENIX_IO_TB_OFFSET           	  0x1C000

#define PHOENIX_CPLD_OFFSET               0xffffffffbd840000ULL

/* Base Address (Virtual) of the PCI Config address space
 * For now, choose 256M phys in kseg1 = 0xA0000000 + (1<<28)
 * Config space spans 256 (num of buses) * 256 (num functions) * 256 bytes
 * ie 1<<24 = 16M
 */ 
#define DEFAULT_PCI_CONFIG_BASE         0x18000000
#define DEFAULT_HT_TYPE0_CFG_BASE       0x16000000
#define DEFAULT_HT_TYPE1_CFG_BASE       0x17000000

#ifndef __ASSEMBLY__

#include <xen/types.h>
#include <asm/byteorder.h>

typedef volatile __u32 register_t;

extern unsigned long mmio_baseaddr;

#define mmio_address(offset) ((register_t *)(mmio_baseaddr + (offset) + HDR_OFFSET))

#define read_register(base, offset) (be32_to_cpu((base)[(offset)]))
#define write_register(base, offset, value) ((base)[(offset)] = cpu_to_be32((value)))

#define __read_register_le32(base, offset) (le32_to_cpu((base)[(offset)]))
#define __write_register_le32(base, offset, value) ((base)[(offset)] = cpu_to_le32((value)))

#define read_register_dw(base, offset) (*((unsigned long *)base + offset))

#define write_register_dw(base, offset, value) \
do { \
*((unsigned long *)base + offset) = value; \
} while (0)

#endif

#endif
