#ifndef _ASM_NETL_PLATFORM_H
#define _ASM_NETL_PLATFORM_H

#include <xen/types.h>

#define RMI_PHOENIX_BOARD_ARIZONA_I   1
#define RMI_PHOENIX_BOARD_ARIZONA_II  2
#define RMI_PHOENIX_BOARD_ARIZONA_III 3
#define RMI_PHOENIX_BOARD_ARIZONA_IV  4
#define RMI_PHOENIX_BOARD_ARIZONA_V   5
#define RMI_PHOENIX_BOARD_ARIZONA_VI   6  /* XLS boards */
#define RMI_PHOENIX_BOARD_ARIZONA_VII 7   /*XLS 2xx boards*/
#define RMI_PHOENIX_BOARD_ARIZONA_VIII 8  /*XLS LTE boards*/
#define RMI_PHOENIX_BOARD_ARIZONA_XI 11
#define RMI_PHOENIX_BOARD_ARIZONA_XII  12

#define XLR_REVISION_A0 0xc0000
#define XLR_REVISION_A1 0xc0001
#define XLR_REVISION_B0 0xc0002
#define XLR_REVISION_B1 0xc0003
#define XLR_REVISION_B2 0xc0004
#define XLR_REVISION_C0 0xc0005
#define XLR_REVISION_C1 0xc0006
#define XLR_REVISION_C2 0xc0007
#define XLR_REVISION_C3 0xc0008
#define XLR_REVISION_C4 0xc0009

#define XLP_REVISION_A0 0xc0010

extern __u32 netl_board_major_ver;
extern __u32 netl_board_minor_ver;

static __inline__ unsigned int xlr_revision(void)
{
	return read_c0_prid() & 0xff00ff;
}

static __inline__ int xlr_revision_a0(void)
{
	return xlr_revision() == XLR_REVISION_A0;
}

static __inline__ int xlr_revision_b0(void)
{
	return xlr_revision() == XLR_REVISION_B0;
}

static __inline__ int xlr_revision_b1(void)
{
        return xlr_revision() == XLR_REVISION_B1;
}

static __inline__ int xlr_revision_c(void)
{
    return (read_c0_prid() >= XLR_REVISION_C0 && read_c0_prid() <= XLR_REVISION_C4);
}

static __inline__ int xlr_board_atx_i(void)
{
	return netl_board_major_ver == RMI_PHOENIX_BOARD_ARIZONA_I;
}

static __inline__ int xlr_board_atx_ii(void)
{
	return netl_board_major_ver == RMI_PHOENIX_BOARD_ARIZONA_II;
}

static __inline__ int xlr_board_atx_ii_b(void)
{
	return (netl_board_major_ver == RMI_PHOENIX_BOARD_ARIZONA_II)
		&& (netl_board_minor_ver == 1);
}

static __inline__ int xlr_board_atx_iii(void)
{
	return netl_board_major_ver == RMI_PHOENIX_BOARD_ARIZONA_III;
}

static __inline__ int xlr_board_atx_iv(void)
{
	return (netl_board_major_ver == RMI_PHOENIX_BOARD_ARIZONA_IV)
		&& (netl_board_minor_ver == 0);
}

static __inline__ int xlr_board_atx_iv_b(void)
{
	return (netl_board_major_ver == RMI_PHOENIX_BOARD_ARIZONA_IV)
		&& (netl_board_minor_ver == 1);
}

static __inline__ int xlr_board_atx_v(void)
{
	return netl_board_major_ver == RMI_PHOENIX_BOARD_ARIZONA_V;
}

static __inline__ int xlr_board_atx_iii_256(void)
{
	return (netl_board_major_ver == RMI_PHOENIX_BOARD_ARIZONA_III)
		&& (netl_board_minor_ver == 0);
}

static __inline__ int xlr_board_atx_iii_512(void)
{
	return (netl_board_major_ver == RMI_PHOENIX_BOARD_ARIZONA_III)
		&& (netl_board_minor_ver == 1);
}

static __inline__ int xlr_board_atx_v_512(void)
{
	return (netl_board_major_ver == RMI_PHOENIX_BOARD_ARIZONA_V)
		&& (netl_board_minor_ver == 1);
}

static __inline__ int xlr_board_atx_vi(void)
{
	return (netl_board_major_ver == RMI_PHOENIX_BOARD_ARIZONA_VI);
}

static __inline__ int xlr_board_atx_vii(void)
{
	return (netl_board_major_ver == RMI_PHOENIX_BOARD_ARIZONA_VII);
}

static __inline__ int xlr_board_atx_viii(void)
{
	return (netl_board_major_ver == RMI_PHOENIX_BOARD_ARIZONA_VIII);
}

static __inline__ int xlr_board_atx_xi(void)
{
	return (netl_board_major_ver == RMI_PHOENIX_BOARD_ARIZONA_XI);
}

static __inline__ int xlr_board_atx_xii(void)
{
	return (netl_board_major_ver == RMI_PHOENIX_BOARD_ARIZONA_XII);
}

#define CHIP_PROCESSOR_ID_XLS_608   0x80
#define CHIP_PROCESSOR_ID_XLS_408   0x88
#define CHIP_PROCESSOR_ID_XLS_404   0x8c
#define CHIP_PROCESSOR_ID_XLS_208   0x8e
#define CHIP_PROCESSOR_ID_XLS_204   0x8f

/* Defines for XLS B0*/
#define CHIP_PROCESSOR_ID_XLS_616_B0   0x40
#define CHIP_PROCESSOR_ID_XLS_608_B0   0x4a
#define CHIP_PROCESSOR_ID_XLS_416_B0   0x44
#define CHIP_PROCESSOR_ID_XLS_412_B0   0x4c
#define CHIP_PROCESSOR_ID_XLS_408_B0   0x4e
#define CHIP_PROCESSOR_ID_XLS_404_B0   0x4f

#define CHIP_PROCESSOR_ID_XLR_B_308   0x06
#define CHIP_PROCESSOR_ID_XLR_B_508   0x07
#define CHIP_PROCESSOR_ID_XLR_B_516   0x08
#define CHIP_PROCESSOR_ID_XLR_B_532   0x09
#define CHIP_PROCESSOR_ID_XLR_B_716   0x0a
#define CHIP_PROCESSOR_ID_XLR_B_732   0x0b

#define CHIP_PROCESSOR_ID_XLR_C_308   0x0F
#define CHIP_PROCESSOR_ID_XLR_C_508   0x0b
#define CHIP_PROCESSOR_ID_XLR_C_516   0x0a
#define CHIP_PROCESSOR_ID_XLR_C_532   0x08
#define CHIP_PROCESSOR_ID_XLR_C_716   0x02

#if defined(CONFIG_RMI_XLP)
/* Fake Values for bring-up */
#define CHIP_PROCESSOR_ID_XLP_A_832   0x00
#define CHIP_PROCESSOR_ID_XLR_C_732   0xff
#else
/* Real Values */
#define CHIP_PROCESSOR_ID_XLP_A_832   0x90
#define CHIP_PROCESSOR_ID_XLR_C_732   0x00
#endif


#endif /* ifndef _ASM_NETL_PLATFORM_H */

