/*
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright (C) 1994, 95, 96, 97, 98, 99, 2000, 2003 Ralf Baechle
 * Copyright (C) 1999, 2000, 2001 Silicon Graphics, Inc.
 */
#ifndef _ASM_PGTABLE_64_H
#define _ASM_PGTABLE_64_H

#include <xen/linkage.h>

#include <asm/addrspace.h>
#include <asm/cachectl.h>

#ifdef CONFIG_XEN_2LEVEL_PGTABLE
#include <asm-generic/pgtable-nopmd.h>
#endif

#include <asm-generic/pgtable-nopud.h>
#include <asm/page.h>

#define PTE_SHIFT DOM_PAGE_SHIFT

#ifndef CONFIG_XEN_2LEVEL_PGTABLE

#define PMD_SHIFT      (PTE_SHIFT + (PAGE_SHIFT + PTE_ORDER - 3)) /* 3 == log(sizeof(pmd_t)) */
#define PMD_SIZE       (1UL << PMD_SHIFT)
#define PMD_MASK       (~(PMD_SIZE-1))

#define PGDIR_SHIFT    (PMD_SHIFT + (PAGE_SHIFT + PMD_ORDER - 3))
#else
#define PGDIR_SHIFT	(PTE_SHIFT + (PAGE_SHIFT + PTE_ORDER - 3)) /* 3 == log(sizeof(pgd_t)) */
#endif

#define PGDIR_SIZE	(1UL << PGDIR_SHIFT)
#define PGDIR_MASK	(~(PGDIR_SIZE-1))

#ifdef CONFIG_XEN_PAGE_SIZE_4KB
#define PGD_ORDER		1
#define PUD_ORDER		aieeee_attempt_to_allocate_pud
#define PMD_ORDER		aieeee_attempt_to_allocate_pmd
#define PTE_ORDER		0
#endif
#ifdef CONFIG_XEN_PAGE_SIZE_8KB
#define PGD_ORDER		0
#define PUD_ORDER		aieeee_attempt_to_allocate_pud
#define PTE_ORDER		0
#endif
#ifdef CONFIG_XEN_PAGE_SIZE_16KB
#define PGD_ORDER		0
#define PUD_ORDER		aieeee_attempt_to_allocate_pud
#define PTE_ORDER		0
#endif
#ifdef CONFIG_XEN_PAGE_SIZE_64KB
#define PGD_ORDER		0
#define PUD_ORDER		aieeee_attempt_to_allocate_pud
#define PTE_ORDER		0
#endif

#ifdef CONFIG_XEN_2LEVEL_PGTABLE
#define PMD_ORDER		aieeee_attempt_to_allocate_pmd
#else
#define PMD_ORDER		0
#endif

#define PTRS_PER_PGD	((PAGE_SIZE << PGD_ORDER) / sizeof(pgd_t))
#define PTRS_PER_PTE	((PAGE_SIZE << PTE_ORDER) / sizeof(pte_t))

#define FIRST_USER_ADDRESS	0UL

#define pte_ERROR(e) \
	printk("%s:%d: bad pte %016lx.\n", __FILE__, __LINE__, pte_val(e))
#define pgd_ERROR(e) \
	printk("%s:%d: bad pgd %016lx.\n", __FILE__, __LINE__, pgd_val(e))

#ifndef CONFIG_XEN_2LEVEL_PGTABLE

#define PTRS_PER_PMD   ((PAGE_SIZE << PMD_ORDER) / sizeof(pmd_t))
#define pmd_ERROR(e) \
       printk("%s:%d: bad pmd %016lx.\n", __FILE__, __LINE__, pmd_val(e))

#endif

extern pte_t invalid_pte_table[PTRS_PER_PTE];
extern pte_t empty_bad_page_table[PTRS_PER_PTE];
extern pmd_t invalid_pmd_table[PTRS_PER_PMD];
extern pmd_t empty_bad_pmd_table[PTRS_PER_PMD];

/*
 * Empty pgd/pmd entries point to the invalid_pte_table.
 */
static inline int pmd_none(pmd_t pmd)
{
    return pmd_val(pmd) == (unsigned long) invalid_pte_table;
}

#define pmd_bad(pmd)        (pmd_val(pmd) & ~PAGE_MASK)

static inline int pmd_present(pmd_t pmd)
{
    return pmd_val(pmd) != (unsigned long) invalid_pte_table;
}

static inline void pmd_clear(pmd_t *pmdp)
{
    pmd_val(*pmdp) = ((unsigned long) invalid_pte_table);
}

#ifndef CONFIG_XEN_2LEVEL_PGTABLE
/*
 * Empty pud entries point to the invalid_pmd_table.
 */
static inline int pud_none(pud_t pud)
{
       return (pud_val(pud) == (unsigned long)invalid_pmd_table);
}
 
static inline int pud_bad(pud_t pud)
{
       return pud_val(pud) & ~PAGE_MASK;
}

static inline int pud_present(pud_t pud)
{
       return (pud_val(pud) != (unsigned long)invalid_pmd_table);
}

static inline void pud_clear(pud_t *pudp)
{
       pud_val(*pudp) = (unsigned long)invalid_pmd_table;
}
#endif


#define PFN_SHIFT 12

// #define pte_page(x)		pfn_to_page(pte_pfn(x))
#define pte_pfn(x)		((unsigned long)(((x).pte >> PFN_SHIFT)))
#define pfn_pte(pfn, prot)	__pte((pfn << PAGE_SHIFT) | pgprot_val(prot))

#define __pgd_offset(address)	pgd_index(address)
#define __pud_offset(address)	(((address) >> PUD_SHIFT) & (PTRS_PER_PUD-1))
#define __pmd_offset(address)	pmd_index(address)

#define pgd_index(address)	(((address) >> PGDIR_SHIFT) & (PTRS_PER_PGD-1))
#define pmd_index(address)	(((address) >> PMD_SHIFT) & (PTRS_PER_PMD-1))

/* to find an entry in a page-table-directory */
#define pgd_offset(mm,addr)	((mm)->pgd + pgd_index(addr))

#ifndef CONFIG_XEN_2LEVEL_PGTABLE

#define pud_phys(pud)		virt_to_phys((void *)pud_val(pud))

static inline unsigned long pud_page_vaddr(pud_t pud)
{
       return pud_val(pud);
}

#define pud_page(pud)          (pfn_to_page(pud_phys(pud) >> PAGE_SHIFT))
 
/* Find an entry in the second-level page table.. */
static inline pmd_t *pmd_offset(pud_t * pud, unsigned long address)
{
       return (pmd_t *) pud_page_vaddr(*pud) + pmd_index(address);
}

#endif /* #ifndef CONFIG_XEN_2LEVEL_PGTABLE */

/* Find an entry in the third-level page table.. */
#define __pte_offset(address)						\
	(((address) >> PTE_SHIFT) & (PTRS_PER_PTE - 1))
#define pte_offset(dir, address)					\
	((pte_t *) pmd_page_vaddr(*(dir)) + __pte_offset(address))
#define pte_offset_map(dir, address)					\
	((pte_t *)page_address(pmd_page(*(dir))) + __pte_offset(address))

#define pte_unmap(pte) ((void)(pte))
#define pte_unmap_nested(pte) ((void)(pte))

/*
 * Non-present pages:  high 24 bits are offset, next 8 bits type,
 * low 32 bits zero.
 */
static inline pte_t mk_swap_pte(unsigned long type, unsigned long offset)
{ pte_t pte; pte_val(pte) = (type << 32) | (offset << 40); return pte; }

#define __swp_type(x)		(((x).val >> 32) & 0xff)
#define __swp_offset(x)		((x).val >> 40)
#define __swp_entry(type,offset) ((swp_entry_t) { pte_val(mk_swap_pte((type),(offset))) })
#define __pte_to_swp_entry(pte)	((swp_entry_t) { pte_val(pte) })
#define __swp_entry_to_pte(x)	((pte_t) { (x).val })

/*
 * Bits 0, 4, 6, and 7 are taken. Let's leave bits 1, 2, 3, and 5 alone to
 * make things easier, and only use the upper 56 bits for the page offset...
 */
#define PTE_FILE_MAX_BITS	56

#define pte_to_pgoff(_pte)	((_pte).pte >> 8)
#define pgoff_to_pte(off)	((pte_t) { ((off) << 8) | _PAGE_FILE })

#endif /* _ASM_PGTABLE_64_H */
